// INCLUDE HEADER ////////////////////////////////////////////////////////

// See header file for licensing info
#include "bt848.h"

// GLOBAL VARS ///////////////////////////////////////////////////////////

int fd, res;

// FUNCTIONS /////////////////////////////////////////////////////////////

// Video source
int tuner_tunerdev() {
    int current;
    int asrc;
    int vsrc;
    current = tuner_videosource();
    if (current < 0) {
        return -1;
    }
    for (vsrc = 0; vsrc < 5; vsrc++) {
        if (tuner_videosource_set(vsrc) < 0) {
            return -1;
        }
        asrc = tuner_audiosource();
        if (asrc < 0) {
            return -1;
        }
        if (asrc != 1) {
            tuner_videosource_set(current);
            return vsrc;
        }
    }
    return -1;
}

int tuner_videosource_set(int videosource) {
    if (videosource < 0 || videosource > 4) {
        return -1;
    }
    if (videosource == 0) {
        videosource = METEOR_DEV0;
    }
    if (videosource == 1) {
        videosource = METEOR_DEV1;
    }
    if (videosource == 2) {
        videosource = METEOR_DEV2;
    }
    if (videosource == 3) {
        videosource = METEOR_DEV3;
    }
    if (videosource == 4) {
        videosource = METEOR_DEV_SVIDEO;
    }
    if ((fd = open(TUNERDEV, O_WRONLY)) < 0) {
        return -1;
    }
    res =  ioctl(fd, METEORSINPUT, &videosource);
    close(fd);
    return res;
}

int tuner_videosource() {
    unsigned long videosource;
    if ((fd = open(TUNERDEV, O_RDONLY)) < 0) {
        return -1;
    }
    if (ioctl(fd, METEORGINPUT, &videosource) < 0) {
        close(fd);
        return -1;
    }
    close(fd);
    if (videosource == METEOR_DEV0) {
        videosource = 0;
    }
    if (videosource == METEOR_DEV1) {
        videosource = 1;
    }
    if (videosource == METEOR_DEV2) {
        videosource = 2;
    }
    if (videosource == METEOR_DEV3) {
        videosource = 3;
    }
    if (videosource == METEOR_DEV_SVIDEO) {
        videosource = 4;
    }
    return (int)videosource;
}

// Audio source
int tuner_audiosource_set(int audiosource) {
    if (audiosource != AUDIO_TUNER && 
        audiosource != AUDIO_INTERN &&
        audiosource != AUDIO_EXTERN &&
        audiosource != AUDIO_MUTE &&
        audiosource != AUDIO_UNMUTE) {
        return -1;
    }
    if ((fd = open(TUNERDEV, O_WRONLY)) < 0) {
        return -1;
    }
    res = ioctl(fd, BT848_SAUDIO, &audiosource);
    close(fd);
    return res;
}

int tuner_audiosource() {
    int audiosource;
    if ((fd = open(TUNERDEV, O_RDONLY)) < 0) {
        return -1;
    }
    if (ioctl(fd, BT848_GAUDIO, &audiosource) < 0) {
        close(fd);
        return -1;
    }
    close(fd);
    return audiosource;
}

// AFC
int tuner_afc_set(int afcbool) {
    if (afcbool != 0 && afcbool != 1) {
        return -1;
    }
    if ((fd = open(TUNERDEV, O_WRONLY)) < 0) {
        return -1;
    }
    res = ioctl(fd, TVTUNER_SETAFC, &afcbool);
    close(fd);
    return res;
}

int tuner_afc() {
    int afcbool;
    if ((fd = open(TUNERDEV, O_RDONLY)) < 0) {
        return -1;
    }
    if (ioctl(fd, TVTUNER_GETAFC, &afcbool) < 0) {
        close(fd);
        return -1;
    }
    close(fd);
    if (afcbool != 1) {
        afcbool = 0;
    }
    return afcbool;
}

// Frequency MHz
int tuner_frequency_set(int frequency) {
    if (frequency > MAXFREQ) {
        frequency = MAXFREQ;
    }
    if (frequency < MINFREQ) {
        frequency = MINFREQ;
    }
    frequency = frequency * FREQFACTOR;
    if ((fd = open(TUNERDEV, O_WRONLY)) < 0) {
        return -1;
    }
    res = ioctl(fd, TVTUNER_SETFREQ, &frequency);
    close(fd);
    return res;
}

int tuner_frequency() {
    int frequency;
    if ((fd = open(TUNERDEV, O_RDWR)) < 0) {
        return -1;
    }
    if (ioctl(fd, TVTUNER_GETFREQ, &frequency) < 0) {
        close(fd);
        return -1;
    }
    close(fd);
    return (frequency / FREQFACTOR);
}

// Brightness, contrast, color, saturation %
int tuner_brightness_set(int brightness) {
    if (brightness > MAXPERC) {
        brightness = MAXPERC;
    }
    if (brightness < MINPERC) {
        brightness = MINPERC;
    }
    brightness = (BT848_BRIGHTMIN + brightness *
        (BT848_BRIGHTMAX - BT848_BRIGHTMIN) / MAXPERC);
    if ((fd = open(TUNERDEV, O_WRONLY)) < 0) {
        return -1;
    }
    res = ioctl(fd, BT848_SBRIG, &brightness);
    close(fd);
    return res;
}

int tuner_brightness() {
    int brightness;
    if ((fd = open(TUNERDEV, O_RDONLY)) < 0) {
        return -1;
    }
    if (ioctl(fd, BT848_GBRIG, &brightness) < 0) {
        close(fd);
        return -1;
    }
    close(fd);
    return (brightness + BT848_BRIGHTMAX);
}

int tuner_contrast_set(int contrast) {
    if (contrast > MAXPERC) {
        contrast = MAXPERC;
    }
    if (contrast < MINPERC) {
        contrast = MINPERC;
    }
    contrast = contrast * BT848_CONTRASTMAX / MAXPERC;
    if ((fd = open(TUNERDEV, O_WRONLY)) < 0) {
        return -1;
    }
    res = ioctl(fd, BT848_SCONT, &contrast);
    close(fd);
    return res;
}

int tuner_contrast() {
    int contrast;
    if ((fd = open(TUNERDEV, O_RDONLY)) < 0) {
        return -1;
    }
    if (ioctl(fd, BT848_GCONT, &contrast) < 0) {
        close(fd);
        return -1;
    }
    close(fd);
    return contrast * MAXPERC / BT848_CONTRASTMAX;
}

int tuner_color_set(int color) {
    if (color > MAXPERC) {
        color = MAXPERC;
    }
    if (color < MINPERC) {
        color = MINPERC;
    }
    color = color * BT848_SATVMAX / MAXPERC;
    if ((fd = open(TUNERDEV, O_WRONLY)) < 0) {
        return -1;
    }
    res = ioctl(fd, BT848_SVSAT, &color) & ioctl(fd, BT848_SHUE, HUEVAL);
    close(fd);
    return res;
}

int tuner_color() {
    int color;
    if ((fd = open(TUNERDEV, O_RDONLY)) < 0) {
        return -1;
    }
    if (ioctl(fd, BT848_GVSAT, &color) < 0) {
        close(fd);
        return -1;
    }
    close(fd);
    return (color + 1) * MAXPERC / BT848_SATVMAX;
}

int tuner_saturation_set(int saturation) {
    if (saturation > MAXPERC) {
        saturation = MAXPERC;
    }
    if (saturation < MINPERC) {
        saturation = MINPERC;
    }
    saturation = saturation * BT848_SATVMAX / MAXPERC;
    if ((fd = open(TUNERDEV, O_WRONLY)) < 0) {
        return -1;
    }
    res = ioctl(fd, BT848_SUSAT, &saturation) & ioctl(fd, BT848_SHUE, HUEVAL);
    close(fd);
    return res;
}

int tuner_saturation() {
    int saturation;
    if ((fd = open(TUNERDEV, O_RDONLY)) < 0) {
        return -1;
    }    
    if (ioctl(fd, BT848_GUSAT, &saturation) < 0) {
        return -1;
    }
    close(fd);
    return (saturation + 1) * MAXPERC / BT848_SATVMAX;
}

// END ///////////////////////////////////////////////////////////////////
