// INCLUDES //////////////////////////////////////////////////////////////

#include <fcntl.h>
#include <unistd.h>
#include <sys/types.h>

#include "pwcbsd/videodev.h"
#include "pwcbsd/pwc-ioctl.h"

// SWIG MODULE DECLARATION ///////////////////////////////////////////////

#ifdef SWIG
%module pwc
%feature("autodoc", "1");
#define MODULE_AUTHOR       "Danny Pansters"
#define MODULE_AUTHOR_EMAIL "danny@ricin.com"
#define MODULE_COPYRIGHT    "(c) 2006, Danny Pansters <danny@ricin.com>\n(c) 1999-2006, Nemosoft Unv, Luc Saillard, Raaf (pwc driver)"
#define MODULE_LICENSE      "BSD"
#endif

// CONVENIENCE CONSTANTS /////////////////////////////////////////////////

#define MINPERC              0  // Lowest percentage value
#define MAXPERC            100  // Highest percentage value
#define PWC_MAXVAL      0xffff  // Max value 65535 (brightness, contrast, ..)
#define PWC_MINVAL           0
#define PWC_MINFPS           4  // Frames per sec
#define PWC_MAXFPS          30

// FUNCTIONS /////////////////////////////////////////////////////////////

// Brightness, contrast, gamma, color, saturation (percentage)
#ifdef SWIG
%feature("docstring", "Set brightness, 0 to 100. Returns -1 on failure.");
#endif
int camera_brightness_set(int fd, int brightness);

#ifdef SWIG
%feature("docstring", "Get brightness. Returns percentage, or -1 on failure.");
#endif
int camera_brightness(int fd);

#ifdef SWIG
%feature("docstring", "Set contrast, 0 to 100. Returns -1 on failure.");
#endif
int camera_contrast_set(int fd, int contrast);

#ifdef SWIG
%feature("docstring", "Get contrast. Returns percentage, or -1 on failure.");
#endif
int camera_contrast(int fd);

#ifdef SWIG
%feature("docstring", "Set gamma, 0 (none) to 100. Returns -1 on failure.");
#endif
int camera_gamma_set(int fd, int gamma);

#ifdef SWIG
%feature("docstring", "Get gamma. Returns percentage, or -1 on failure.");
#endif
int camera_gamma(int fd);

#ifdef SWIG
%feature("docstring", "Set color (whitebalance), 0 (green) to 100 (max blue & red). Returns -1 on failure.");
#endif
int camera_color_set(int fd, int color);

#ifdef SWIG
%feature("docstring", "Get color (whitebalance). Returns percentage, or -1 on failure.");
#endif
int camera_color(int fd);

#ifdef SWIG
%feature("docstring", "Set saturation, 0 to 100. Returns -1 on failure.");
#endif
int camera_saturation_set(int fd, int saturation);

#ifdef SWIG
%feature("docstring", "Get saturation. Returns percentage, or -1 on failure.");
#endif
int camera_saturation(int fd);

// Framerate
#ifdef SWIG
%feature("docstring", "Set framerate, 4 to 30. Returns -1 on failure.");
#endif
int camera_framerate_set(int fd, int fps);

#ifdef SWIG
%feature("docstring", "Get framerate. Returns 4 to 30, or -1 on failure.");
#endif
int camera_framerate(int fd);

// Capture size
#ifdef SWIG
%feature("docstring", "Set capture size, 0 to 5 (SQCIF, QSIF, QCIF, SIF, CIF, VGA). Returns -1 on failure.");
#endif
int camera_capsize_set(int fd, int capsize);

#ifdef SWIG
%feature("docstring", "Get capture size. Returns 0 to 5, or -1 on failure.");
#endif
int camera_capsize(int fd);

// MPT
#ifdef SWIG
%feature("docstring", "Perform panning, 0 to 100 (50 is center). Returns -1 on failure.");
#endif
int camera_panning_set(int fd, int pan);

#ifdef SWIG
%feature("docstring", "Get panning. Returns 0 to 100, or -1 on failure.");
#endif
int camera_panning(int fd);

#ifdef SWIG
%feature("docstring", "Perform tilting, 0 to 100 (50 is center). Returns -1 on failure.");
#endif
int camera_tilting_set(int fd, int tilt);

#ifdef SWIG
%feature("docstring", "Get tilting. Returns 0 to 100, or -1 on failure.");
#endif
int camera_tilting(int fd);

// END ///////////////////////////////////////////////////////////////////
