// INCLUDE HEADERS ///////////////////////////////////////////////////////

// See header file for licensing info
#include "saa.h"

#include <unistd.h>

// GLOBAL VARS ///////////////////////////////////////////////////////////

int fd, res;
int curfreq = 0;
int muted = 0;
int asrc = 0;
struct saa_video_opt vo;
static struct saa_audio_spec audio_spec = {
    // saa/tvv/FLTK/audioopt.c
    .sas_audsel = AUDSEL_SIF,
    .sas_sfs = SFS_SIF_32KHZ,
    .sas_icsch = 1,

    .sas_cap_sampcount = 16384,
    .sas_audfmt = AUDFMT_16BIT,
    .sas_audsamp = AUDSAMP_STEREO,
    .sas_flag_samp2complement = 1,
    .sas_flags =  0,
};

// FUNCTIONS /////////////////////////////////////////////////////////////

// Video source
int tuner_tunerdev() {
    // Fixed value, always 1
    return 1;
}

int tuner_videosource_set(int videosource) {
    /* VSRC                  MAPPING (like bt848)
    
    MODE_CVBS_1 = 0           DEV0 (ext0)
    MODE_CVBS_2 = 1           DEV1 (tuner)
    MODE_CVBS_3 = 2           DEV2 (ext2)
    MODE_CVBS_4 = 3           DEV3 (ext3)
    MODE_CVBS_5               ignored
    MODE_SVIDEOFIXGC_1        DEV4/SVID (svideo / fixed gain)
    MODE_SVIDEOFIXGC_2        ignored
    MODE_SVIDEOAGC_1          ignored
    MODE_SVIDEOAGC_2          ignored
    */
    if (videosource < 0 || videosource > 4) {
        return -1;
    }
    if ((fd = open(SAADEV, O_RDWR, 0)) < 0) {
        return -1;
    }
    res = ioctl(fd, GETVIDEOOPT, &vo);
    if (res < 0) {
        close(fd);
        return -1;
    }
    if (videosource == 0) {
        vo.svo_input_mode = MODE_CVBS_1;
        asrc = 1;
    }
    if (videosource == 1) {
        // tuner
        vo.svo_input_mode = MODE_CVBS_2;
        asrc = 0;
    }
    if (videosource == 2) {
        vo.svo_input_mode = MODE_CVBS_3;
        asrc = 1;
    }
    if (videosource == 3) {
        vo.svo_input_mode = MODE_CVBS_4;
        asrc = 1;
    }
    if (videosource == 4) {
        // svideo (first)
        vo.svo_input_mode = MODE_SVIDEOFIXGC_1;
        asrc = 1;
    }
    res = ioctl(fd, SETVIDEOOPT, &vo);
    close(fd);
    tuner_audiosource_set(asrc);
    return res;
}

int tuner_videosource() {
    if ((fd = open(SAADEV, O_RDWR, 0)) < 0) {
        return -1;
    }
    res = ioctl(fd, GETVIDEOOPT, &vo);
    close(fd);
    if (res < 0) {
        return -1;
    }
    if (vo.svo_input_mode == MODE_CVBS_1) {
        return 0;
    }
    if (vo.svo_input_mode == MODE_CVBS_2) {
        return 1;
    }
    if (vo.svo_input_mode == MODE_CVBS_3) {
        return 2;
    }
    if (vo.svo_input_mode == MODE_CVBS_4) {
        return 3;
    }
    if (vo.svo_input_mode == MODE_SVIDEOFIXGC_1) {
        return 4;
    }
    return -1;
}

// Audio init
int tuner_audio_init() {
    int foo;
    if ((fd = open(SAUDEV, O_RDWR, 0)) < 0) {
        return -1;
    }
    if (ioctl(fd, ACAPTUREON, &audio_spec) != 0) {
        close(fd);
        return -1;
    }
    usleep(100000);
    if (ioctl(fd, ACAPTUREOFF, &foo) != 0) {
        close(fd);
        return -1;
    }
    close(fd);
    return 0;
}

// Audio source
int tuner_audiosource_set(int audiosource) {
    /*   saa              context             bt848                    mapto
    OCS_DACLEFT_DACRIGHT  tuner to soundcard  AUDIO_TUNER   tuner to sc   0
    OCS_INLEFT1_INRIGHT1  from line-in        AUDIO_EXTERN  line-in to sc 1
    OCS_INLEFT2_INRIGHT2  other line-in       AUDIO_INTERN  ...           2
    others are ignored

    Emulate bt muting behaviour, if muted the asrc returned must be 0x80
    */
    if ((fd = open(SAUDEV, O_RDWR, 0)) < 0) {
        return -1;
    }
    if (audiosource == 0) {
        audiosource = OCS_DACLEFT_DACRIGHT;
        asrc = audiosource;
    }
    else if (audiosource == 1) {
        audiosource = OCS_INLEFT1_INRIGHT1;
        asrc = audiosource;
    }
    else if (audiosource == 2) {
        audiosource = OCS_INLEFT2_INRIGHT2;
        asrc = audiosource;
    }
    else {
        if (audiosource == 0x80) {
            // res = ioctl(fd, SETAUDIOMUTE, );
            // THIS NEEDS SOME WORK
            
            // fake mute by using OCS_INLEFT2_INRIGHT2 that's likely unused
            audiosource = OCS_INLEFT2_INRIGHT2;
            muted = 1;
        }
        else if (audiosource == 0x81) {
            audiosource = asrc;
            muted = 0;
        }
        else {
            close(fd);
            return -1;
        }
        
    }
    res = ioctl(fd, SETAUDIOMIXER, &audiosource);
    close(fd);
    return res;
}

int tuner_audiosource() {
    if (muted == 1) {
        return 0x80;
    }
    enum OCS audiosource;
    if ((fd = open(SAUDEV, O_RDWR, 0)) < 0) {
        return -1;
    }
    if (ioctl(fd, GETAUDIOMIXER, &audiosource) != 0) {
        close(fd);
        return -1;
    }
    close(fd);
    if (audiosource == OCS_DACLEFT_DACRIGHT) {
        return 0;
    }
    if (audiosource == OCS_INLEFT1_INRIGHT1) {
        return 1;
    }
    if (audiosource == OCS_INLEFT2_INRIGHT2) {
        return 2;
    }
    return -1;
}

// AFC
int tuner_afc_set(int afcbool) {
    // No AFC
    return -1;
}

int tuner_afc() {
    // No AFC
    return 0;
}

// Frequency MHz
int tuner_frequency_set(int frequency) {
    if (frequency == curfreq) {
        return 0;
    }
    if (frequency > MAXFREQ) {
        frequency = MAXFREQ;
    }
    if (frequency < MINFREQ) {
        frequency = MINFREQ;
    }
    if ((fd = open(IICDEV, O_RDWR, 0)) < 0) {
        return -1;
    }
    res = set_channel(fd, frequency, curfreq, 0);
    close(fd);
    if (res == 0) {
        curfreq = frequency;
    }
    return res;
}

int tuner_frequency() {
    // There's no frequency retrieval, so we keep track with curfreq
    return curfreq;
}

// Brightness, contrast, color, saturation %
int tuner_brightness_set(int brightness) {
    if (brightness > MAXPERC) {
        brightness = MAXPERC;
    }
    if (brightness < MINPERC) {
        brightness = MINPERC;
    }
    brightness = brightness * SAA_BRIGHTMAX / MAXPERC;
    if ((fd = open(SAADEV, O_RDWR, 0)) < 0) {
        return -1;
    }
    res = ioctl(fd, GETVIDEOOPT, &vo);
    if (res < 0) {
        close(fd);
        return -1;
    }
    vo.svo_brightness = brightness;
    res = ioctl(fd, SETVIDEOOPT, &vo);
    close(fd);
    return res;
}

int tuner_brightness() {
    if ((fd = open(SAADEV, O_RDWR, 0)) < 0) {
        return -1;
    }
    res = ioctl(fd, GETVIDEOOPT, &vo);
    close(fd);
    if (res < 0) {
        return -1;
    }
    return vo.svo_brightness * MAXPERC / SAA_BRIGHTMAX;
}

int tuner_contrast_set(int contrast) {
    if (contrast > MAXPERC) {
        contrast = MAXPERC;
    }
    if (contrast < MINPERC) {
        contrast = MINPERC;
    }
    contrast = contrast * SAA_CONTMAX / MAXPERC;
    if ((fd = open(SAADEV, O_RDWR, 0)) < 0) {
        return -1;
    }
    res = ioctl(fd, GETVIDEOOPT, &vo);
    if (res < 0) {
        close(fd);
        return -1;
    }
    vo.svo_contrast = contrast;
    res = ioctl(fd, SETVIDEOOPT, &vo);
    close(fd);
    return res;
}

int tuner_contrast() {
    if ((fd = open(SAADEV, O_RDWR, 0)) < 0) {
        return -1;
    }
    res = ioctl(fd, GETVIDEOOPT, &vo);
    close(fd);
    if (res < 0) {
        return -1;
    }
    return vo.svo_contrast * MAXPERC / SAA_CONTMAX;
}

int tuner_color_set(int color) {
    if (color >= MAXPERC || color < MINPERC) {
        color = MINPERC;
    }
    if (color <= 50) {
        color = SAA_HUEMAX * (50 - color) / MAXPERC;
    }
    else {
        color = SAA_HUEMAX * (color - 50) / MAXPERC;
    }
    if ((fd = open(SAADEV, O_RDWR, 0)) < 0) {
        return -1;
    }
    res = ioctl(fd, GETVIDEOOPT, &vo);
    if (res < 0) {
        close(fd);
        return -1;
    }
    vo.svo_hue = color;
    res = ioctl(fd, SETVIDEOOPT, &vo);                        
    close(fd);
    return res;
}

int tuner_color() {
    if ((fd = open(SAADEV, O_RDWR, 0)) < 0) {
        return -1;
    }
    res = ioctl(fd, GETVIDEOOPT, &vo);
    close(fd);
    if (res < 0) {
        return -1;
    }
    if (vo.svo_hue <= 128) {
        return (50 - vo.svo_hue * MAXPERC / SAA_HUEMAX);
    }
    return vo.svo_hue * MAXPERC / SAA_HUEMAX;
}

int tuner_saturation_set(int saturation) {
    if (saturation > MAXPERC) {
        saturation = MAXPERC;
    }
    if (saturation < MINPERC) {
        saturation = MINPERC;
    }
    saturation = saturation * SAA_SATMAX / MAXPERC;
    if ((fd = open(SAADEV, O_RDWR, 0)) < 0) {
        return -1;
    }
    res = ioctl(fd, GETVIDEOOPT, &vo);
    if (res < 0) {
        close(fd);
        return -1;
    }
    vo.svo_saturation = saturation;
    res = ioctl(fd, SETVIDEOOPT, &vo);
    close(fd);
    return res;
}

int tuner_saturation() {
    int saturation;
    if ((fd = open(SAADEV, O_RDWR, 0)) < 0) {
        return -1;
    }
    res = ioctl(fd, GETVIDEOOPT, &vo);
    close(fd);
    if (res < 0) {
        return -1;
    }
    close(fd);
    return vo.svo_saturation * MAXPERC / SAA_SATMAX;
}

// END ///////////////////////////////////////////////////////////////////
